function df = fetch_series(varargin)
% function fetch_series(varargin)
% Download time series from DBnomics. 
% Returns a cell array.
%
% Examples:
% Fetch one series:
% fetch_series('provider_code', 'IMF', 'dataset_code', 'CPI', 'series_code', 'M.FR+DE.PCPIEC_IX+PCPIA_IX');
% fetch_series('provider_code', 'IMF', 'dataset_code', 'CPI', 'series_code', '.FR.PCPIEC_WT');
% 
% Fetch all the series of a dataset:
% fetch_series('provider_code', 'AMECO', 'dataset_code', 'UVGD', 'max_nb_series', 500);
% 
% Fetch many series from different datasets:
% fetch_series('series_ids', {'AMECO/ZUTN/EA19.1.0.0.0.ZUTN', 'AMECO/ZUTN/DNK.1.0.0.0.ZUTN', 'IMF/CPI/A.AT.PCPIT_IX'});
% 
% Fetch many series from the same dataset, searching by dimension:
% fetch_series('provider_code','AMECO', 'dataset_code', 'ZUTN', 'dimensions', '{"geo":["dnk"]}');
% 
% POSSIBLE PARAMETERS
%   provider_code           [string]     the code of the dataset provider.  
%   dataset_code            [string]     the code of the dataset.  
%   series_code             [string]     the code mask of the series. If provided, the provider_code and the dataset_code must specified.         
%   dimensions              [char]       dataset dimension codes. If provided it must be a string formatted like: '{"country":["ES","FR","IT"],"indicator":["IC.REG.COST.PC.FE.ZS.DRFN"]}'.
%   series_ids              [string]     list of series IDs. It is string formatted like `provider_code/dataset_code/series_code`. If provided, the provider_code and the dataset_code shouldn't be specified.
%   max_nb_series           [integer]    maximum number of series requested by the API. If not provided, a default value of 50 series will be used.
%   api_base_url            [string]     the base URL used for API requests. If not provided, a default value of: 'https://api.db.nomics.world/v22/' will be used.       
%   dbnomics_filters        [char]       filters to apply on the requested series. If provided it must be a string formatted like: '[{"code": "interpolate", "parameters": {"frequency": "monthly", "method": "spline"}}]'.
%
% OUTPUTS
%   df
%
% SPECIAL REQUIREMENTS
%   When you (don't) use dimensions, you must specifiy provider_code and dataset_code.
%   When you use series_code, you must specifiy provider_code and dataset_code.
%   When you use series_ids, you must not specifiy provider_code nor dataset_code.

% Copyright (C) 2020 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

default_api_base_url = 'https://api.db.nomics.world/v22/'; 
default_editor_base_url = 'https://editor.nomics.world/api/v1/';

p = inputParser;
validStringInput = @(x) isstring(x) || ischar(x) || iscellstr(x);
p.addParameter('provider_code', '', validStringInput);
p.addParameter('dataset_code', '', validStringInput);
p.addParameter('series_code', '', validStringInput); 
p.addParameter('dimensions', @ischar);
p.addParameter('series_ids', '',validStringInput);
p.addParameter('max_nb_series', NaN, @isnumeric);
p.addParameter('api_base_url', default_api_base_url, validStringInput);
p.addParameter('dbnomics_filters', '', @ischar);
p.KeepUnmatched = false;
p.parse(varargin{:});

if strcmp(p.Results.api_base_url(end),'/') == 0
    p.Results.api_base_url = [p.Results.api_base_url '/'];
end

if isempty(p.Results.dataset_code)
    if iscell(p.Results.provider_code)
        p.Results.series_ids = p.Results.provider_code;
        p.Results.provider_code = '';
    elseif ~isempty(p.Results.provider_code)
        p.Results.series_ids = {p.Results.provider_code};
        p.Results.provider_code = '';
    end
end

series_base_url = [p.Results.api_base_url 'series'];

if isa(p.Results.dimensions, 'function_handle') && isempty(p.Results.series_code) && isempty(p.Results.series_ids)
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you don''t use dimensions, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s?observations=1', series_base_url, p.Results.provider_code, p.Results.dataset_code);
end

if ~isa(p.Results.dimensions, 'function_handle')
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you use dimensions, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s?observations=1&dimensions=%s', series_base_url, p.Results.provider_code, p.Results.dataset_code, p.Results.dimensions); %jsonencode(
end

if ~isempty(p.Results.series_code)
    if isempty(p.Results.provider_code) || isempty(p.Results.dataset_code)
        error('When you use series_code, you must specifiy provider_code and dataset_code.');
    end
    api_link = sprintf('%s/%s/%s/%s?observations=1', series_base_url, p.Results.provider_code, p.Results.dataset_code, p.Results.series_code);
end

if ~isempty(p.Results.series_ids)
    if ~isempty(p.Results.provider_code) || ~isempty(p.Results.dataset_code)
        error('When you use series_ids, you must not specifiy provider_code nor dataset_code.');
    end
    if iscellstr(p.Results.series_ids)
        series_ids = strjoin(p.Results.series_ids,',');
    else
        series_ids = p.Results.series_ids;
    end
    api_link = sprintf('%s?observations=1&series_ids=%s', series_base_url, series_ids);
end
df = fetch_series_by_api_link(api_link, p.Results.dbnomics_filters, p.Results.max_nb_series, default_editor_base_url);
end

